@@ -26,12 +26,13 @@ class WikiServlet < WEBrick::HTTPServlet::AbstractServlet
 		app = KakiRequestHandler.new($page_data_source, 'kaki');
 		app.theme_dir = $theme_dir
 		app.theme_base_url = $theme_base_url
-		header, body = app.main(req.header, req.query)
+		header, body = app.main(req.meta_vars, req.query)
 		header.each(){ |key, value|
 			res[key] = value
 		}
 		res.body =  body
 	end
+
 	def do_POST(req, res)
 		do_GET(req, res)
 	end
