@@ -2,8 +2,14 @@
 #$data_dir  = '/var/tmp/wiki_data/'
 $data_dir  = './data/'
 
+# access log file path
+$log_file = './access.log'
+
+# theme
 $theme_dir  = './theme/'
 $theme_base_url  = 'theme/'
 
+# cache option
 $use_cache = false
 $use_memory_cache = false
+
