@@ -27,7 +27,7 @@ class KakiRequestHandler
 	attr 'theme_dir', true
 	attr 'theme_base_url', true
 
-	def main(request_header, request_parameters)
+	def main(request_variables, request_parameters)
 		command = request_command(request_parameters)
 
 		data = {}
@@ -38,37 +38,39 @@ class KakiRequestHandler
 			# command 
 			case command
 			when 'create' then
-				create(request_header, request_parameters, data)
+				create(request_variables, request_parameters, data)
+			when 'property' then
+				property(request_variables, request_parameters, data)
 			when 'edit' then
-				edit(request_header, request_parameters, data)
+				edit(request_variables, request_parameters, data)
 			when 'preview' then
-				preview(request_header, request_parameters, data)
+				preview(request_variables, request_parameters, data)
 			when 'save' then
-				save(request_header, request_parameters, data)
+				save(request_variables, request_parameters, data)
 			when 'change_passwd' then
 				tmpl_file = "save.html"
-				change_passwd(request_header, request_parameters, data)
+				change_passwd(request_variables, request_parameters, data)
 			when 'snapshot' then
 				tmpl_file = "save.html"
-				snapshot(request_header, request_parameters, data)
+				snapshot(request_variables, request_parameters, data)
 			when 'index' then
-				index(request_header, request_parameters, data)
+				index(request_variables, request_parameters, data)
 			when 'recent' then
-				recent(request_header, request_parameters, data)
+				recent(request_variables, request_parameters, data)
 			when 'admin' then
-				admin(request_header, request_parameters, data)
+				admin(request_variables, request_parameters, data)
 			when 'delete' then
-				delete(request_header, request_parameters, data)
+				delete(request_variables, request_parameters, data)
 			else 
 				# default command is 'view'
 				command = 'view'
 				tmpl_file = 'view.html'
-				view(request_header, request_parameters, data)
+				view(request_variables, request_parameters, data)
 			end
 
 
 			# side menu
-			request_lang_list = self.reuqest_language_list(request_header)
+			request_lang_list = self.reuqest_language_list(request_variables)
 			sub_page_id = @page_data_source.suitable_page_id('SideMenu', request_lang_list)
 			if sub_page_id then
 				sub_page_name, sub_page_lang, = @page_data_source.parse_page_id(sub_page_id)
@@ -150,6 +152,8 @@ class KakiRequestHandler
 		end
 =end
 
+#		data[:exception] = CGI.escapeHTML(request_variables.inspect)
+
 		# http response body
 		body = Misen.expand_text(Misen::STYLE_SGML, tmpl, data)
 		return [response_header, body];
@@ -160,11 +164,11 @@ protected
 	attr_reader "site_property"
 	attr_writer "site_property"
 
-	def view(request_header, request_parameters, data)
+	def view(request_variables, request_parameters, data)
 		request_page_id = request_page_id(request_parameters, 'FrontPage')
 		page_name, lang, version = @page_data_source.parse_page_id(request_page_id)
 
-		request_lang_list = self.reuqest_language_list(request_header)
+		request_lang_list = self.reuqest_language_list(request_variables)
 
 		if lang then
 			if @page_data_source.exist?(page_name, lang, version) then
@@ -188,18 +192,12 @@ protected
 				passwd = property['passwd']
 				data[:is_frozen] = self.site_property['is_frozen']
 				data[:is_frozen] = true if (passwd and passwd.length > 0)
+				data[:is_not_frozen] = (not data[:is_frozen])
 				data[:counter] = { :count => property['count_int']}
 				data[:source_time] = property['source_time']
 				data[:page_title] = CGI.escapeHTML(property['page_title'])
 				data[:modified_time] = property['source_time'].to_data_hash
-				if request_header['referer'] then
-					url = request_header['referer'].to_s 
-					referers = property['referers'].to_s.split(", ")
-					referers.delete(url)
-					referers.unshift(url)
-					referers = referers[0,10]
-					property['referers'] = referers.join(", ")
-				end
+				append_referer(request_variables, property)
 				property['count_int'] += 1
 			}
 			
@@ -217,11 +215,52 @@ protected
 		end
 	end
 
-	def create(request_header, request_parameters, data)
+	def create(request_variables, request_parameters, data)
 		data[:language_list] = make_language_list_data(supported_language_list())
 	end
 
-	def edit(request_header, request_parameters, data)
+	def property(request_variables, request_parameters, data)
+		request_page_id = request_page_id(request_parameters, 'FrontPage')
+		page_name, lang, version = @page_data_source.parse_page_id(request_page_id)
+
+		request_lang_list = self.reuqest_language_list(request_variables)
+
+		if lang then
+			if @page_data_source.exist?(page_name, lang, version) then
+				page_id = request_page_id
+			end
+		else
+			page_id = @page_data_source.suitable_page_id(page_name, request_lang_list, version)
+		end
+
+		raise "NoSuchPage" unless page_id 
+
+		page_name, page_lang, page_version = @page_data_source.parse_page_id(page_id)
+		property = @page_data_source.page_property(page_name, page_lang, page_version)
+		passwd = property['passwd']
+		data[:is_frozen] = self.site_property['is_frozen']
+		data[:is_frozen] = true if (passwd and passwd.length > 0)
+		data[:is_not_frozen] = (not data[:is_frozen])
+		data[:counter] = { :count => property['count_int']}
+		data[:source_time] = property['source_time']
+		data[:page_title] = CGI.escapeHTML(property['page_title'])
+		data[:modified_time] = property['source_time'].to_data_hash
+		data[:page_id_uri] = URI.escape(page_id)
+		data[:page_id] = CGI.escapeHTML(page_id)
+		data[:page_name] = CGI.escapeHTML(page_name)
+		data[:page_lang] = CGI.escapeHTML(page_lang)
+		data[:page_name_and_lang] = URI.escape(@page_data_source.make_page_id(page_name, page_lang))
+		if page_version then
+			data[:page_version] = { :version_int => page_version.to_i } 
+		end
+
+		data[:history] = page_history_data(page_name, page_lang)
+		data[:lang_selector] = page_language_selector_data(page_name, page_lang)
+		data[:referer] = page_referer_data(property)
+	end
+
+
+	def edit(request_variables, request_parameters, data)
 		page_id = self.request_page_id(request_parameters)
 		raise 'NoPageID' unless page_id
 		page_name, page_lang, page_version = @page_data_source.parse_page_id(page_id)
@@ -236,7 +275,7 @@ protected
 		end
 
 		page_id = @page_data_source.make_page_id(page_name, page_lang)
-		page_title = page_id
+		page_title = page_name
 
 		content_source = ''
 		source_time_str = '0'
@@ -268,7 +307,7 @@ protected
 		data[:edit_key] = source_time_str.crypt(salt)
 	end
 
-	def preview(request_header, request_parameters, data)
+	def preview(request_variables, request_parameters, data)
 		page_id = request_page_id(request_parameters)
 		content_source = request_content_source(request_parameters)
 		edit_key = request_edit_key(request_parameters)
@@ -295,14 +334,15 @@ protected
 		data[:page_title] = CGI.escapeHTML(page_title)
 
 		data[:is_frozen] = is_frozen
+		data[:is_not_frozen] = (not is_frozen)
 		data[:content_source] = CGI.escapeHTML(content_source)
 		data[:edit_key] = edit_key
 
 		wiki_doc =  Wiki::PageParser.new().parse(content_source);
 		if wiki_doc then
 			@wiki2html.output = ''
-#			@wiki2html.default_lang_list = [page_lang] + self.reuqest_language_list(request_header)
-#			@wiki2html.default_lang_list = self.reuqest_language_list(request_header)
+#			@wiki2html.default_lang_list = [page_lang] + self.reuqest_language_list(request_variables)
+#			@wiki2html.default_lang_list = self.reuqest_language_list(request_variables)
 			@wiki2html.default_lang_list = nil
 			wiki_doc.accept(@wiki2html)
 			data[:content] = @wiki2html.output 
@@ -311,7 +351,7 @@ protected
 		end
 	end
 
-	def save(request_header, request_parameters, data)
+	def save(request_variables, request_parameters, data)
 		page_id = request_page_id(request_parameters)
 		content_source = request_content_source(request_parameters)
 		edit_key = request_edit_key(request_parameters)
@@ -353,7 +393,7 @@ protected
 		end
 	end
 
-	def change_passwd(request_header, request_parameters, data)
+	def change_passwd(request_variables, request_parameters, data)
 		passwd = request_parameters['passwd'].to_s.untaint
 		new_passwd1 = request_parameters['new_passwd1'].to_s.untaint
 		new_passwd2 = request_parameters['new_passwd1'].to_s.untaint
@@ -395,7 +435,7 @@ protected
 		end
 	end
 
-	def snapshot(request_header, request_parameters, data)
+	def snapshot(request_variables, request_parameters, data)
 		page_id = request_page_id(request_parameters)
 		passwd = request_passwd(request_parameters)
 		page_name, page_lang, = @page_data_source.parse_page_id(page_id)
@@ -429,7 +469,7 @@ protected
 
 	end
 
-	def index(request_header, request_parameters, data)
+	def index(request_variables, request_parameters, data)
 		pages = @page_data_source.page_list
 		page_data_by_page_name = {}
 		pages.each{ |page_info|
@@ -461,6 +501,7 @@ protected
 					:page_lang => CGI.escapeHTML(lang),
 					:page_title => CGI.escapeHTML(page_property['page_title']),
 					:is_frozen => is_frozen, 
+					:is_not_frozen => (not is_frozen), 
 					:count => page_property['count_int'],
 					:version_list => [],
 				}
@@ -486,7 +527,7 @@ protected
 		data[:index] ={ :page_list => page_list}
 	end
 
-	def recent(request_header, request_parameters, data)
+	def recent(request_variables, request_parameters, data)
 		pages = @page_data_source.page_list
 		pages.sort!(){ |a, b|
 			b[1]['source_time'] <=> a[1]['source_time']
@@ -514,6 +555,7 @@ protected
 			page_data[:page_lang] = CGI.escapeHTML(lang)
 			page_data[:is_frozen] = self.site_property['is_frozen']
 			page_data[:is_frozen] = true if (page_property['passwd'].to_s.length > 0)
+			page_data[:is_not_frozen] = (not page_data[:is_frozen])
 			if version then
 				page_data[:version] = { :version_int => page_property['version_int'] }
 			else
@@ -530,7 +572,7 @@ protected
 		data[:recent] ={ :date_list => date_list }
 	end
 
-	def admin_save_property(request_header, request_parameters, data)
+	def admin_save_property(request_variables, request_parameters, data)
 		admin_passwd = request_parameters['admin_passwd'].to_s.untaint
 		site_name = request_parameters['site_name'].to_s.untaint
 		admin_name = request_parameters['admin_name'].to_s.untaint
@@ -559,7 +601,7 @@ protected
 		}
 	end
 
-	def admin_change_passwd(request_header, request_parameters, data)
+	def admin_change_passwd(request_variables, request_parameters, data)
 		admin_passwd = request_parameters['admin_passwd'].to_s.untaint
 		new_passwd1 = request_parameters['new_admin_passwd1'].to_s.untaint
 		new_passwd2 = request_parameters['new_admin_passwd2'].to_s.untaint
@@ -584,7 +626,7 @@ protected
 		}
 	end
 
-	def admin_site_freeze(request_header, request_parameters, data)
+	def admin_site_freeze(request_variables, request_parameters, data)
 		admin_passwd = request_parameters['admin_passwd'].to_s.untaint
 		site_freeze = request_parameters['site_freeze'].to_s.untaint
 		@page_data_source.site_property(){ |property|
@@ -604,16 +646,16 @@ protected
 		}
 	end
 
-	def admin(request_header, request_parameters, data)
+	def admin(request_variables, request_parameters, data)
 		save_property = request_parameters['save_property'].to_s.untaint
 		change_passwd = request_parameters['change_passwd'].to_s.untaint
 		site_freeze = request_parameters['site_freeze'].to_s.untaint
 		if save_property.length > 0 then
-			admin_save_property(request_header, request_parameters, data)
+			admin_save_property(request_variables, request_parameters, data)
 		elsif change_passwd.length > 0 then
-			admin_change_passwd(request_header, request_parameters, data)
+			admin_change_passwd(request_variables, request_parameters, data)
 		elsif site_freeze.length > 0 then
-			admin_site_freeze(request_header, request_parameters, data)
+			admin_site_freeze(request_variables, request_parameters, data)
 		end
 
 		theme_list = Dir.glob("#{self.theme_dir}/*/").sort().collect(){ |theme_dir|
@@ -644,7 +686,7 @@ protected
 		data[:languages] = self.site_property['languages']
 	end
 
-	def delete_page(request_header, request_parameters, data)
+	def delete_page(request_variables, request_parameters, data)
 		admin_passwd = request_parameters['admin_passwd'].to_s.untaint
 		@page_data_source.site_property(){ |property|
 			if admin_login_ok?(property, admin_passwd) then
@@ -662,10 +704,10 @@ protected
 		}
 	end
 
-	def delete(request_header, request_parameters, data)
+	def delete(request_variables, request_parameters, data)
 		delete_page = request_parameters['delete_page'].to_s.untaint
 		if delete_page.length > 0 then
-			delete_page(request_header, request_parameters, data)
+			delete_page(request_variables, request_parameters, data)
 		end
 
 		page_list = []
@@ -729,8 +771,8 @@ protected
 		return result;
 	end
 
-	def reuqest_language_list(request_header)
-		result = parse_accept_language(request_header['accept-language'].to_s.untaint)
+	def reuqest_language_list(request_variables)
+		result = parse_accept_language(request_variables['HTTP_ACCEPT_LANGUAGE'].to_s.untaint)
 		result.push(self.site_property["default_language"].untaint)
 		return result;
 	end
@@ -808,12 +850,15 @@ protected
 		lang_list = []
 		@page_data_source.page_list().each(){ |page|
 			id = page[0]
+			prop = page[1]
 			n, l, v= @page_data_source.parse_page_id(page[0])
 			if n == page_name and v == nil then
 				is_selected = false
 				is_selected = true if page_lang == l 
 				lang_list << {
-					:page_id_uri=> URI.escape("#{n}.#{l}"),
+					:page_id_uri => URI.escape("#{n}.#{l}"),
+					:page_id => CGI.escapeHTML("#{n}.#{l}"),
+					:page_title => CGI.escapeHTML(prop['page_title']),
 					:is_selected => is_selected,
 					:lang => l,
 				}
@@ -828,6 +873,19 @@ protected
 		return result;
 	end
 
+	def page_referer_data(page_property)
+		result = nil
+		referers = page_property['referers'].to_s.split(", ")
+		referer_list = referers.collect(){ |referer|
+			{ :referer_url => referer }
+		}
+
+		if referer_list.length > 0 then
+			result = { :referer_list => referer_list }
+		end
+		result
+	end
+
 	def make_language_list_data(language_list)
 		result = []
 		default_language = self.site_property['default_language'] 
@@ -868,6 +926,37 @@ protected
 		[result, lang]
 	end
 
+	def append_referer(request_variables, page_property)
+		referer = request_variables['HTTP_REFERER']
+		
+		if referer then
+			request_uri = URI.parse(request_variables['REQUEST_URI'])
+			if request_uri.host then
+				server_uri = request_uri.host
+				server_uri += ":#{request_uri.port}" if request_uri.port != 80
+			else
+				server_uri = request_variables['HTTP_HOST']
+			end
+			server_uri += request_uri.path
+
+			url_str = referer.to_s 
+			uri = URI.parse(url_str)
+			referer_uri = uri.host
+			referer_uri += ":#{uri.port}" if uri.port != 80
+			referer_uri += uri.path
+			if server_uri != referer_uri then
+				referers = page_property['referers'].to_s.split(", ")
+				referers.delete(url_str)
+				referers.delete_if() { |ref|
+					ref =~ /#{Regexp.escape(server_uri)}/ 
+				}
+				referers.unshift(url_str)
+				referers = referers[0,10]
+				page_property['referers'] = referers.join(", ")
+			end
+		end
+	end
+
 	def page_login_ok?(page_property, passwd)
 		result = false
 
