@@ -2,7 +2,7 @@
 
 $: << "../src"
 
-require 'WikiPageParser.rb'
+require 'PageParser.rb'
 require 'ToHtml.rb'
 
 if $0 == __FILE__ then
@@ -31,13 +31,12 @@ if $0 == __FILE__ then
 			wiki_html = ''
 			doc.accept(Wiki::ToHtml.new(wiki_html))
 			outfile << <<EOD
-<?xml version="1.0" encoding="euc-jp" ?>
-<!DOCTYPE html
-PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<?xml version="1.0" encoding="UTF8" ?>
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" lang="ja" xml:lang="ja" >
 <head>
-	<meta http-equiv="Content-Type" content="text/html; charset=euc-jp" />
+	<meta http-equiv="Content-Type" content="text/html; charset=UTF8" />
 	<meta name="ROBOTS" content="ALL" />
 	<title>
 		#{basename}
